@@ -4,7 +4,10 @@ mod analyze;
 mod deno_comment_directives;
 mod deno_globals;
 mod module_specifiers;
+mod polyfill;
 
+pub use analyze::get_ignore_line_indexes;
 pub use deno_comment_directives::*;
 pub use deno_globals::*;
 pub use module_specifiers::*;
+pub use polyfill::*;
